@@ -1,7 +1,6 @@
 // Licensed GNU LGPL v3 or later: http://www.gnu.org/licenses/lgpl.html
 
 #include <assert.h>
-#include <sys/time.h>
 #include <stdio.h>
 
 #include <vector>
@@ -17,17 +16,22 @@ using namespace SpectMorph;
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   QApplication app (argc, argv);
 
-  if (argc != 2)
+  const char *index;
+  if (argc == 1)
+    index = "instruments:standard";
+  else if (argc == 2)
+    index = argv[1];
+  if (argc > 2)
     {
       printf ("usage: %s <smindex-file>\n", argv[0]);
       exit (1);
     }
 
-  NavigatorWindow window (argv[1]);
+  NavigatorWindow window (index);
   window.show();
 
   return app.exec();
