@@ -29,8 +29,6 @@ SpectrumWindow::SpectrumWindow (Navigator *navigator)
   zoom_controller->set_hscrollbar (scroll_area->horizontalScrollBar());
   zoom_controller->set_vscrollbar (scroll_area->verticalScrollBar());
   connect (zoom_controller, SIGNAL (zoom_changed()), this, SLOT (on_zoom_changed()));
-  connect (navigator->display_param_window(), SIGNAL (params_changed()),
-           spectrum_view, SLOT (on_display_params_changed()));
 }
 
 void
