@@ -8,6 +8,7 @@
 #include "smutils.hh"
 #include "smblockutils.hh"
 #include "smalignedarray.hh"
+#include "smrandom.hh"
 
 #include <math.h>
 #include <stdio.h>
@@ -16,6 +17,7 @@
 #include <complex>
 #include <map>
 #include <algorithm>
+#include <memory>
 #include <cinttypes>
 
 using namespace SpectMorph;
@@ -44,19 +46,33 @@ normalize_phase (double phase)
 #define debug(...) SpectMorph::Debug::debug ("encoder", __VA_ARGS__)
 
 EncoderParams::EncoderParams() :
-  mix_freq (0),
-  frame_step_ms (0),
-  frame_size_ms (0),
-  zeropad (0),
-  frame_step (0),
-  frame_size (0),
-  block_size (0),
-  fundamental_freq (0),
   param_name_d ({"peak-width", "min-frame-periods", "min-frame-size"}),
   param_name_s ({"window"})
 {
 }
 
+bool
+EncoderParams::add_config_entry (const string& param, const string& value)
+{
+  for (auto name : param_name_d) // double parameters
+    {
+      if (name == param)
+        {
+          param_value_d[name] = sm_atof (value.c_str());
+          return true;
+        }
+    }
+  for (auto name : param_name_s) // string parameters
+    {
+      if (name == param)
+        {
+          param_value_s[name] = value;
+          return true;
+        }
+    }
+  return false; // unsupported parameter
+}
+
 bool
 EncoderParams::load_config (const std::string& filename)
 {
@@ -168,6 +184,59 @@ EncoderParams::get_param (const string& param, string& value) const
     }
 }
 
+static size_t
+make_odd (size_t n)
+{
+  if (n & 1)
+    return n;
+  return n - 1;
+}
+
+void
+EncoderParams::setup_params (const WavData& wav_data, double new_fundamental_freq)
+{
+  mix_freq         = wav_data.mix_freq();
+  zeropad          = 4;
+  fundamental_freq = new_fundamental_freq;
+
+  // --- frame size & step ---
+  double min_frame_periods, min_frame_size;
+  if (!get_param ("min-frame-periods", min_frame_periods))
+    min_frame_periods = 4;  // default: at least 4 periods of the fundamental per frame
+  if (!get_param ("min-frame-size", min_frame_size))
+    min_frame_size = 40;    // default: at least 40ms frames
+
+  frame_size_ms = min_frame_size;
+  frame_size_ms = max<float> (frame_size_ms, 1000 / fundamental_freq * min_frame_periods);
+  frame_step_ms = frame_size_ms / 4.0;
+
+  // --- convert block sizes in ms to sample counts ----
+  frame_size = make_odd (mix_freq * 0.001 * frame_size_ms);
+  frame_step = mix_freq * 0.001 * frame_step_ms;
+
+  /* compute block size from frame size (smallest 2^k value >= frame_size) */
+  block_size = 1;
+  while (block_size < frame_size)
+    block_size *= 2;
+
+  /* compute encoder window */
+  window.resize (block_size);
+
+  for (size_t i = 0; i < window.size(); i++)
+    {
+      if (i < frame_size)
+        window[i] = window_cos (2.0 * i / (frame_size - 1) - 1.0);
+      else
+        window[i] = 0;
+    }
+}
+
+void
+EncoderParams::set_kill_function (const std::function<bool()>& new_kill_function)
+{
+  kill_function = new_kill_function;
+}
+
 /**
  * Constructor which initializes the Encoders parameters.
  */
@@ -181,6 +250,7 @@ Encoder::Encoder (const EncoderParams& enc_params)
   assert (enc_params.frame_size > 0);
   assert (enc_params.block_size > 0);
   assert (enc_params.fundamental_freq > 0);
+  assert (enc_params.window.size() == enc_params.block_size);
 
   this->enc_params = enc_params;
 
@@ -191,34 +261,12 @@ Encoder::Encoder (const EncoderParams& enc_params)
   optimal_attack.attack_end_ms = 0;
 }
 
-bool
-Encoder::check_harmonic (double freq, double& new_freq, double mix_freq)
-{
-  if (enc_params.fundamental_freq > 0)
-    {
-      for (int i = 1; i < 100; i++)
-	{
-          // FIXME: why hardcode 2048
-	  double base_freq = (mix_freq/2048*16);
-	  double harmonic_freq = i * base_freq;
-	  double diff = fabs (harmonic_freq - freq) / base_freq;
-	  if (diff < 0.125)
-	    {
-	      new_freq = harmonic_freq;
-	      return true;
-	    }
-	}
-    }
-  new_freq = freq;
-  return false;
-}
-
 /**
  * This function computes the short-time-fourier-transform (STFT) of the input
  * signal using a window to cut the individual frames out of the sample.
  */
 void
-Encoder::compute_stft (const WavData& multi_channel_wav_data, int channel, const vector<float>& window)
+Encoder::compute_stft (const WavData& multi_channel_wav_data, int channel)
 {
   /* deinterleave multi channel signal */
   vector<float> single_channel_signal;
@@ -229,7 +277,7 @@ Encoder::compute_stft (const WavData& multi_channel_wav_data, int channel, const
 
   original_samples = single_channel_signal;
 
-  WavData wav_data (single_channel_signal, 1, multi_channel_wav_data.mix_freq());
+  WavData wav_data (single_channel_signal, 1, multi_channel_wav_data.mix_freq(), multi_channel_wav_data.bit_depth());
 
   /* encode single channel */
   zero_values_at_start = enc_params.frame_size - enc_params.frame_step / 2;
@@ -237,10 +285,11 @@ Encoder::compute_stft (const WavData& multi_channel_wav_data, int channel, const
 
   wav_data.prepend (zero_values);
 
-  const uint64 n_values = wav_data.n_values();
+  const uint64 n_values   = wav_data.n_values();
   const size_t frame_size = enc_params.frame_size;
   const size_t block_size = enc_params.block_size;
   const int    zeropad    = enc_params.zeropad;
+  const auto&  window     = enc_params.window;
 
   sample_count = n_values;
 
@@ -281,6 +330,9 @@ Encoder::compute_stft (const WavData& multi_channel_wav_data, int channel, const
       audio_block.debug_samples.assign (debug_samples.begin(), debug_samples.begin() + frame_size);
 
       audio_blocks.push_back (audio_block);
+
+      if (killed ("_stft", audio_blocks.size() & 63))
+        break; // break to avoid leaking fft_in, fft_out
     }
   FFT::free_array_float (fft_in);
   FFT::free_array_float (fft_out);
@@ -318,12 +370,13 @@ public:
  * This function searches for peaks in the frame ffts. These are stored in frame_tracksels.
  */
 void
-Encoder::search_local_maxima (const vector<float>& window)
+Encoder::search_local_maxima()
 {
   const size_t block_size = enc_params.block_size;
   const size_t frame_size = enc_params.frame_size;
   const int    zeropad    = enc_params.zeropad;
   const double mix_freq   = enc_params.mix_freq;
+  const auto&  window     = enc_params.window;
 
   // figure out normalization for window
   double window_weight = 0;
@@ -439,12 +492,10 @@ Encoder::search_local_maxima (const vector<float>& window)
                         }
                       tracksel.phase = phase;
 
-                      double dummy_freq;
-                      tracksel.is_harmonic = check_harmonic (tracksel.freq, dummy_freq, mix_freq);
                       // FIXME: need a different criterion here
                       // mag2 > -30 doesn't track all partials
                       // mag2 > -60 tracks lots of junk, too
-                      if ((mag2 > -90 || tracksel.is_harmonic) && tracksel.freq > 10)
+                      if (mag2 > -90 && tracksel.freq > 10)
                         frame_tracksels[n].push_back (tracksel);
 
                       if (peak_type == PEAK_DOUBLE)
@@ -456,6 +507,9 @@ Encoder::search_local_maxima (const vector<float>& window)
 #endif
             }
 	}
+
+      if (killed ("_maxima", n & 15))
+        return;
     }
 }
 
@@ -570,30 +624,21 @@ Encoder::validate_partials()
   map<Tracksel *, bool> processed_tracksel;
   for (size_t n = 0; n < audio_blocks.size(); n++)
     {
-      vector<Tracksel>::iterator i, j;
+      vector<Tracksel>::iterator i;
       for (i = frame_tracksels[n].begin(); i != frame_tracksels[n].end(); i++)
 	{
 	  if (!processed_tracksel[&(*i)])
 	    {
 	      double biggest_mag = -100;
-	      bool   is_harmonic = false;
-	      for (Tracksel *t = &(*i); t->next; t = t->next)
+	      for (Tracksel *t = &(*i); t; t = t->next)
 		{
 		  biggest_mag = max (biggest_mag, t->mag2);
-		  if (t->is_harmonic)
-		    is_harmonic = true;
 		  processed_tracksel[t] = true;
 		}
-	      if (biggest_mag > -90 || is_harmonic)
+	      if (biggest_mag > -90)
 		{
-		  for (Tracksel *t = &(*i); t->next; t = t->next)
+		  for (Tracksel *t = &(*i); t; t = t->next)
 		    {
-#if 0
-		      double new_freq;
-		      if (check_harmonic (t->freq, new_freq, mix_freq))
-			t->freq = new_freq;
-#endif
-
 		      audio_blocks[t->frame].freqs.push_back (t->freq);
 		      audio_blocks[t->frame].mags.push_back (t->mag);
 		      audio_blocks[t->frame].phases.push_back (t->phase);
@@ -601,6 +646,8 @@ Encoder::validate_partials()
 		}
 	    }
 	}
+      if (killed ("_validate", n & 63))
+        return;
     }
 }
 
@@ -609,11 +656,12 @@ Encoder::validate_partials()
  * residue (remaining energy not corresponding to sine frequencies).
  */
 void
-Encoder::spectral_subtract (const vector<float>& window)
+Encoder::spectral_subtract()
 {
   const size_t block_size = enc_params.block_size;
   const size_t frame_size = enc_params.frame_size;
-  const size_t zeropad = enc_params.zeropad;
+  const size_t zeropad    = enc_params.zeropad;
+  const auto&  window     = enc_params.window;
 
   float *fft_in = FFT::new_array_float (block_size * zeropad);
   float *fft_out = FFT::new_array_float (block_size * zeropad);
@@ -669,6 +717,9 @@ Encoder::spectral_subtract (const vector<float>& window)
 	    }
 	  debug ("finalspectrum:%" PRId64 " %g\n", frame, mag);
 	}
+
+      if (killed ("_subtract", frame & 7))
+        return;
     }
   FFT::free_array_float (fft_in);
   FFT::free_array_float (fft_out);
@@ -873,16 +924,19 @@ remove_small_partials (EncoderBlock& audio_block)
  * in the previous steps.
  */
 void
-Encoder::optimize_partials (const vector<float>& window, int optimization_level)
+Encoder::optimize_partials (int optimization_level)
 {
   const double mix_freq = enc_params.mix_freq;
 
   for (uint64 frame = 0; frame < audio_blocks.size(); frame++)
     {
       if (optimization_level >= 1) // redo FFT estmates, only better
-        refine_sine_params_fast (audio_blocks[frame], mix_freq, frame, window);
+        refine_sine_params_fast (audio_blocks[frame], mix_freq, frame, enc_params.window);
 
       remove_small_partials (audio_blocks[frame]);
+
+      if (killed ("_optimize"))
+        return;
     }
 }
 
@@ -942,7 +996,7 @@ approximate_noise_spectrum (int frame,
     }
 }
 
-void
+static void
 xnoise_envelope_to_spectrum (int frame,
                              double mix_freq,
                              const vector<double>& envelope,
@@ -1016,11 +1070,12 @@ xnoise_envelope_to_spectrum (int frame,
  * for a noise signal.
  */
 void
-Encoder::approx_noise (const vector<float>& window)
+Encoder::approx_noise()
 {
   const size_t block_size = enc_params.block_size;
   const size_t frame_size = enc_params.frame_size;
-  const size_t zeropad = enc_params.zeropad;
+  const size_t zeropad    = enc_params.zeropad;
+  const auto&  window     = enc_params.window;
 
   double sum_w2 = 0;
   for (size_t x = 0; x < frame_size; x++)
@@ -1072,17 +1127,22 @@ Encoder::approx_noise (const vector<float>& window)
       debug ("noiseenergy:%" PRId64 " %f %f %f\n", frame, spect_energy, b4_energy, r_energy);
       /// } DEBUG_CODE
       audio_blocks[frame].noise.assign (noise_envelope.begin(), noise_envelope.end());
+
+      if (killed ("_noise", frame & 7))
+        return;
     }
 }
 
 double
-Encoder::attack_error (const vector< vector<double> >& unscaled_signal, const vector<float>& window, const Attack& attack, vector<double>& out_scale)
+Encoder::attack_error (const vector< vector<double> >& unscaled_signal, const Attack& attack, vector<double>& out_scale)
 {
   const size_t frames = unscaled_signal.size();
   double total_error = 0;
   vector<double> decoded_signal (enc_params.frame_size + enc_params.frame_step * frames);
   vector<double> orig_signal (decoded_signal.size());
 
+  const auto& window = enc_params.window;
+
   for (size_t f = 0; f < frames; f++)
     {
       const vector<double>& frame_signal = unscaled_signal[f];
@@ -1143,8 +1203,10 @@ Encoder::attack_error (const vector< vector<double> >& unscaled_signal, const ve
  * attack envelope (attack_start_ms and attack_end_ms) given the data.
  */
 void
-Encoder::compute_attack_params (const vector<float>& window)
+Encoder::compute_attack_params()
 {
+  int killed_iteration = 0;
+
   const double mix_freq   = enc_params.mix_freq;
   const size_t frame_size = enc_params.frame_size;
   const size_t frames = MIN (20, audio_blocks.size());
@@ -1168,10 +1230,17 @@ Encoder::compute_attack_params (const vector<float>& window)
               frame_signal[n] += sin (phase) * mag;
               phase += f / mix_freq * 2.0 * M_PI;
             }
+
+          if (killed ("__attack", killed_iteration++ & 63))
+            return;
         }
       unscaled_signal.push_back (frame_signal);
     }
 
+  /* make attack envelope deterministically return the same result for the same input every time */
+  Random random;
+  random.set_seed (42);
+
   Attack attack;
   int no_modification = 0;
   double error = 1e7;
@@ -1194,9 +1263,11 @@ Encoder::compute_attack_params (const vector<float>& window)
         R = 0.2;
       else if (no_modification < 2500)
         R = 0.01;
+      else
+        R = 0.002;
 
-      new_attack.attack_start_ms += g_random_double_range (-R, R);
-      new_attack.attack_end_ms += g_random_double_range (-R, R);
+      new_attack.attack_start_ms += random.random_double_range (-R, R);
+      new_attack.attack_end_ms += random.random_double_range (-R, R);
 
       // constrain attack to at least 5ms to avoid clickiness at start
       new_attack.attack_end_ms = max (new_attack.attack_end_ms, new_attack.attack_start_ms + 5);
@@ -1205,7 +1276,7 @@ Encoder::compute_attack_params (const vector<float>& window)
           new_attack.attack_start_ms >= zero_values_at_start_ms &&
           new_attack.attack_end_ms < 200)
         {
-          const double new_error = attack_error (unscaled_signal, window, new_attack, scale);
+          const double new_error = attack_error (unscaled_signal, new_attack, scale);
 #if 0
           printf ("attack=<%f, %f> error=%.17g new_attack=<%f, %f> new_arror=%.17g\n", attack.attack_start_ms, attack.attack_end_ms, error,
                                                                                        new_attack.attack_start_ms, new_attack.attack_end_ms, new_error);
@@ -1222,6 +1293,8 @@ Encoder::compute_attack_params (const vector<float>& window)
               no_modification++;
             }
         }
+      if (killed ("_attack", killed_iteration++ & 63))
+        return;
     }
   for (size_t f = 0; f < frames; f++)
     {
@@ -1284,32 +1357,54 @@ Encoder::sort_freqs()
  * This function calls all steps necessary for encoding in the right order.
  *
  * \param dhandle a data handle containing the signal to be encoded
- * \param window the analysis window
  * \param optimization_level determines if fast (0), medium (1), or very slow (2) algorithm is used
  * \param attack whether to find the optimal attack parameters
  */
-void
-Encoder::encode (const WavData& wav_data, int channel, const vector<float>& window, int optimization_level,
+bool
+Encoder::encode (const WavData& wav_data, int channel, int optimization_level,
                  bool attack, bool track_sines)
 {
-  compute_stft (wav_data, channel, window);
+  compute_stft (wav_data, channel);
+  if (killed ("stft"))
+    return false;
 
   if (track_sines)
     {
-      search_local_maxima (window);
+      search_local_maxima();
+      if (killed ("maxima"))
+        return false;
+
       link_partials();
+      if (killed ("link"))
+        return false;
+
       validate_partials();
+      if (killed ("validate"))
+        return false;
 
-      optimize_partials (window, optimization_level);
+      optimize_partials (optimization_level);
+      if (killed ("optimize"))
+        return false;
 
-      spectral_subtract (window);
+      spectral_subtract();
+      if (killed ("subtract"))
+        return false;
     }
-  approx_noise (window);
+  approx_noise();
+  if (killed ("noise"))
+    return false;
 
   if (attack)
-    compute_attack_params (window);
+    compute_attack_params();
+
+  if (killed ("attack"))
+    return false;
 
   sort_freqs();
+  if (killed ("sort"))
+    return false;
+
+  return true;
 }
 
 void
@@ -1368,7 +1463,9 @@ convert_freqs_mags_phases (const EncoderBlock& eblock, AudioBlock& ablock, const
         {
           ablock.freqs.push_back (ifreq);
           ablock.mags.push_back (imag);
-          ablock.phases.push_back (iphase);
+
+          if (enc_params.enable_phases)
+            ablock.phases.push_back (iphase);
         }
     }
 }
@@ -1385,18 +1482,31 @@ convert_noise (const vector<float>& noise, vector<uint16_t>& inoise)
 /**
  * This function saves the data produced by the encoder to a SpectMorph file.
  */
-void
+Error
 Encoder::save (const string& filename)
 {
-  SpectMorph::Audio audio;
-  audio.fundamental_freq = enc_params.fundamental_freq;
-  audio.mix_freq = enc_params.mix_freq;
-  audio.frame_size_ms = enc_params.frame_size_ms;
-  audio.frame_step_ms = enc_params.frame_step_ms;
-  audio.attack_start_ms = optimal_attack.attack_start_ms;
-  audio.attack_end_ms = optimal_attack.attack_end_ms;
-  audio.zero_values_at_start = zero_values_at_start;
-  audio.zeropad = enc_params.zeropad;
+  std::unique_ptr<Audio> audio (save_as_audio());
+
+  return audio->save (filename);  // saving can fail
+}
+
+/**
+ * This function saves the data produced by the encoder, returning a newly
+ * allocated Audio object (caller must free this).
+ */
+Audio *
+Encoder::save_as_audio()
+{
+  Audio *audio = new Audio();
+
+  audio->fundamental_freq = enc_params.fundamental_freq;
+  audio->mix_freq = enc_params.mix_freq;
+  audio->frame_size_ms = enc_params.frame_size_ms;
+  audio->frame_step_ms = enc_params.frame_step_ms;
+  audio->attack_start_ms = optimal_attack.attack_start_ms;
+  audio->attack_end_ms = optimal_attack.attack_end_ms;
+  audio->zero_values_at_start = zero_values_at_start;
+  audio->zeropad = enc_params.zeropad;
 
   for (vector<EncoderBlock>::iterator ai = audio_blocks.begin(); ai != audio_blocks.end(); ai++)
     {
@@ -1405,27 +1515,27 @@ Encoder::save (const string& filename)
       convert_noise (ai->noise, block.noise);
       block.original_fft = ai->original_fft;
       block.debug_samples = ai->debug_samples;
-      audio.contents.push_back (block);
+      audio->contents.push_back (block);
     }
-  audio.sample_count = sample_count;
-  audio.original_samples = original_samples;
+  audio->sample_count = sample_count;
+  audio->original_samples = original_samples;
   if (loop_start >= 0 && loop_end >= 0 && loop_type != Audio::LOOP_NONE)
     {
-      audio.loop_type = loop_type;
-      audio.loop_start = loop_start;
-      audio.loop_end = loop_end;
+      audio->loop_type = loop_type;
+      audio->loop_start = loop_start;
+      audio->loop_end = loop_end;
 
-      if (audio.loop_type == Audio::LOOP_TIME_FORWARD || audio.loop_type == Audio::LOOP_TIME_PING_PONG)
+      if (audio->loop_type == Audio::LOOP_TIME_FORWARD || audio->loop_type == Audio::LOOP_TIME_PING_PONG)
         {
-          audio.loop_start += zero_values_at_start;
-          audio.loop_end += zero_values_at_start;
+          audio->loop_start += zero_values_at_start;
+          audio->loop_end += zero_values_at_start;
         }
     }
-  audio.save (filename);
+  return audio;
 }
 
 void
-Encoder::debug_decode (const string& filename, const vector<float>& enc_window)
+Encoder::debug_decode (const string& filename)
 {
   const double mix_freq   = enc_params.mix_freq;
   const size_t frame_step = enc_params.frame_step;
