@@ -2,6 +2,7 @@
 
 #include "smleakdebugger.hh"
 #include "smmain.hh"
+#include "smdebug.hh"
 #include <assert.h>
 #include <glib.h>
 
@@ -19,7 +20,7 @@ LeakDebugger::ptr_add (void *p)
     {
       assert (sm_init_done());
 
-      QMutexLocker lock (&mutex);
+      std::lock_guard<std::mutex> lock (mutex);
 
       if (ptr_map[p] != 0)
         g_critical ("LeakDebugger: invalid registration of object type %s detected; ptr_map[p] is %d\n",
@@ -36,7 +37,7 @@ LeakDebugger::ptr_del (void *p)
     {
       assert (sm_init_done());
 
-      QMutexLocker lock (&mutex);
+      std::lock_guard<std::mutex> lock (mutex);
 
       if (ptr_map[p] != 1)
         g_critical ("LeakDebugger: invalid deletion of object type %s detected; ptr_map[p] is %d\n",
@@ -46,8 +47,9 @@ LeakDebugger::ptr_del (void *p)
     }
 }
 
-LeakDebugger::LeakDebugger (const string& name) :
-  type (name)
+LeakDebugger::LeakDebugger (const string& name, std::function<void()> cleanup_function) :
+  type (name),
+  cleanup_function (cleanup_function)
 {
 }
 
@@ -55,6 +57,9 @@ LeakDebugger::~LeakDebugger()
 {
   if (DEBUG)
     {
+      if (cleanup_function)
+        cleanup_function();
+
       int alive = 0;
 
       for (map<void *, int>::iterator pi = ptr_map.begin(); pi != ptr_map.end(); pi++)
@@ -68,6 +73,7 @@ LeakDebugger::~LeakDebugger()
       if (alive)
         {
           g_printerr ("LeakDebugger (%s) => %d objects remaining\n", type.c_str(), alive);
+          sm_debug ("LeakDebugger (%s) => %d objects remaining\n", type.c_str(), alive);
         }
     }
 }
