@@ -179,6 +179,8 @@ LiveDecoder::retrigger (int channel, float freq, int midi_velocity, float mix_fr
       original_sample_pos = 0;
       original_samples_norm_factor = db_to_factor (audio->original_samples_norm_db);
 
+      done_state = DoneState::ACTIVE;
+
       // reset partial state vectors
       pstate[0].clear();
       pstate[1].clear();
@@ -287,6 +289,11 @@ LiveDecoder::process_internal (size_t n_values, float *audio_out, float portamen
 
           original_sample_pos += phase_inc;
         }
+      if (original_sample_pos > audio->original_samples.size() && get_loop_type() != Audio::LOOP_TIME_FORWARD)
+        {
+          if (done_state == DoneState::ACTIVE)
+            done_state = DoneState::ALMOST_DONE;
+        }
       return;
     }
 
@@ -337,7 +344,6 @@ LiveDecoder::process_internal (size_t n_values, float *audio_out, float portamen
               const AudioBlock& audio_block = *audio_block_ptr;
 
               assert (audio_block.freqs.size() == audio_block.mags.size());
-              assert (audio_block.freqs.size() == audio_block.phases.size());
 
               ifft_synth->clear_partials();
 
@@ -486,6 +492,11 @@ LiveDecoder::process_internal (size_t n_values, float *audio_out, float portamen
                   ifft_synth->get_samples (samples, IFFTSynth::ADD);
                 }
             }
+          else
+            {
+              if (done_state == DoneState::ACTIVE)
+                done_state = DoneState::ALMOST_DONE;
+            }
           pos = 0;
           have_samples = block_size / 2;
         }
@@ -664,13 +675,20 @@ LiveDecoder::process (size_t n_values, const float *freq_in, float *audio_out)
   if (!audio)   // nothing loaded
     {
       std::fill (audio_out, audio_out + n_values, 0);
+      done_state = DoneState::DONE;
       return;
     }
+  /* ensure that time_offset_ms() is only called during live decoder process */
+  assert (!in_process);
+  in_process = true;
+  start_env_pos = env_pos;
   /*
    * split processing into small blocks, max 10 ms
    *  -> limit n_values to keep portamento stretch settings up-to-date
    */
   const size_t max_n_values = current_mix_freq * 0.010;
+  const size_t orig_n_values = n_values;
+  const float *orig_audio_out = audio_out;
 
   while (n_values > 0)
     {
@@ -692,6 +710,25 @@ LiveDecoder::process (size_t n_values, const float *freq_in, float *audio_out)
       audio_out += todo_values;
       n_values -= todo_values;
     }
+
+  /* update done state ALMOST_DONE => DONE: we need
+   *  - done state ALMOST_DONE
+   *  - a silent output buffer
+   */
+  if (done_state == DoneState::ALMOST_DONE)
+    {
+      size_t i = 0;
+      while (i < orig_n_values)
+        {
+          if (orig_audio_out[i] != 0.0)
+            break;
+
+          i++;
+        }
+      if (i == orig_n_values)
+        done_state = DoneState::DONE;
+    }
+  in_process = false;
 }
 
 
@@ -819,3 +856,43 @@ LiveDecoder::set_vibrato (bool enabled, float depth, float frequency, float atta
   vibrato_frequency   = frequency;
   vibrato_attack      = attack;
 }
+
+double
+LiveDecoder::current_pos() const
+{
+  if (!audio)
+    return -1;
+
+  if (original_samples_enabled)
+    return original_sample_pos * 1000.0 / audio->mix_freq;
+
+  return frame_idx * audio->frame_step_ms - audio->zero_values_at_start * 1000.0 / audio->mix_freq;
+}
+
+double
+LiveDecoder::fundamental_note() const
+{
+  if (!audio)
+    return -1;
+
+  return freq_to_note (audio->fundamental_freq);
+}
+
+bool
+LiveDecoder::done() const
+{
+  return done_state == DoneState::DONE;
+}
+
+double
+LiveDecoder::time_offset_ms() const
+{
+  /* LiveDecoder::process() produces samples by IFFTSynth - possibly more than once per block
+   *
+   * This function provides the time offset of the IFFTSynth call relative to the start of
+   * the sample block generated by process(). LFOs (and other operators) can use this
+   * information for jitter-free timing.
+   */
+  assert (in_process);
+  return 1000 * (env_pos - start_env_pos) / current_mix_freq;
+}
