@@ -1,6 +1,7 @@
 // Licensed GNU LGPL v3 or later: http://www.gnu.org/licenses/lgpl.html
 
 #include "smwavsetrepo.hh"
+#include "smmain.hh"
 
 using namespace SpectMorph;
 
@@ -9,17 +10,13 @@ using std::string;
 WavSetRepo*
 WavSetRepo::the()
 {
-  static WavSetRepo *instance = NULL;
-  if (!instance)
-    instance = new WavSetRepo;
-
-  return instance;
+  return Global::wav_set_repo();
 }
 
 WavSet*
 WavSetRepo::get (const string& filename)
 {
-  QMutexLocker lock (&mutex);
+  std::lock_guard<std::mutex> lock (mutex);
 
   WavSet*& wav_set = wav_set_map[filename];
   if (!wav_set)
@@ -29,3 +26,9 @@ WavSetRepo::get (const string& filename)
     }
   return wav_set;
 }
+
+WavSetRepo::~WavSetRepo()
+{
+  for (auto w : wav_set_map)
+    delete w.second;
+}
