@@ -10,8 +10,8 @@
 #include "smmain.hh"
 #include "sminfile.hh"
 #include "smutils.hh"
-#include "smlpc.hh"
 #include "smfft.hh"
+#include "smaudiotool.hh"
 
 using namespace SpectMorph;
 using std::vector;
@@ -19,7 +19,6 @@ using std::min;
 using std::max;
 using std::string;
 using std::set;
-using std::complex;
 
 double
 vector_delta (const vector<double>& a, const vector<double>& b)
@@ -126,7 +125,7 @@ load_or_die (Audio& audio, const string& filename, const string& mode)
     load_options = AUDIO_SKIP_DEBUG;
 
   Error error = audio.load (filename, load_options);
-  if (error != 0)
+  if (error)
     {
       fprintf (stderr, "can't load file: %s\n",filename.c_str());
       exit (1);
@@ -284,7 +283,7 @@ public:
   {
     if (args.size() == 1)
       {
-        percent = atof (args[0].c_str());
+        percent = sm_atof (args[0].c_str());
         return true;
       }
     return false;
@@ -667,8 +666,8 @@ public:
   {
     if (args.size() == 2)
       {
-        freq_min = atof (args[0].c_str());
-        freq_max = atof (args[1].c_str());
+        freq_min = sm_atof (args[0].c_str());
+        freq_max = sm_atof (args[1].c_str());
         return true;
       }
     return false;
@@ -802,7 +801,7 @@ public:
   {
     if (args.size() == 1)
       {
-        percent = atof (args[0].c_str());
+        percent = sm_atof (args[0].c_str());
         if (percent < 0 || percent > 100)
           {
             fprintf (stderr, "bad loop percentage: %f\n", percent);
@@ -875,53 +874,12 @@ public:
   bool
   exec (Audio& audio)
   {
-    const double freq_min = 0.8;
-    const double freq_max = 1.25;
-    double freq_sum = 0, mag_sum = 0;
-
-    for (size_t f = 0; f < audio.contents.size(); f++)
+    double tune_factor;
+    if (AudioTool::get_auto_tune_factor (audio, tune_factor))
       {
-        double position_percent = f * 100.0 / audio.contents.size();
-        if (position_percent >= 40 && position_percent <= 60)
-          {
-            const AudioBlock& block = audio.contents[f];
-            double best_freq = -1;
-            double best_mag = 0;
-            for (size_t n = 0; n < block.freqs.size(); n++)
-              {
-                const double freq = block.freqs_f (n);
+        AudioTool::apply_auto_tune_factor (audio, tune_factor);
 
-                if (freq > freq_min && freq < freq_max)
-                  {
-                    const double m = block.mags_f (n);
-                    if (m > best_mag)
-                      {
-                        best_mag = m;
-                        best_freq = block.freqs_f (n);
-                      }
-                  }
-              }
-            if (best_mag > 0)
-              {
-                freq_sum += best_freq * best_mag;
-                mag_sum += best_mag;
-              }
-          }
-      }
-    if (mag_sum > 0)
-      {
-        double tune_factor = 1.0 / (freq_sum / mag_sum);
         double input_fundamental_freq = audio.fundamental_freq / tune_factor;
-        for (size_t f = 0; f < audio.contents.size(); f++)
-          {
-            AudioBlock& block = audio.contents[f];
-
-            for (size_t n = 0; n < block.freqs.size(); n++)
-              {
-                const double freq = block.freqs_f (n) * tune_factor;
-                block.freqs[n] = sm_freq2ifreq (freq);
-              }
-          }
         sm_printf ("%.17g  %.17g  %.3f cent\n", audio.fundamental_freq, input_fundamental_freq, freq_ratio_to_cent (tune_factor));
 
         set_need_save (true);
@@ -1020,26 +978,44 @@ public:
   }
 } tune_all_frames_command;
 
-static void
-normalize_factor (double norm, Audio& audio)
+class SmoothTuneCommand : public Command
 {
-  const int    norm_delta_idb   = sm_factor2delta_idb (norm);
+  int fundamental_est_n; /* number of partials to use for fundamental estimation in range [1,3] */
+  double smooth_ms;
+  double smooth_percent;
+public:
+  SmoothTuneCommand() : Command ("smooth-tune")
+  {
+  }
+  bool
+  parse_args (vector<string>& args)
+  {
+    if (args.size() == 3)
+      {
+        fundamental_est_n = atoi (args[0].c_str());
+        assert (fundamental_est_n >= 1 && fundamental_est_n <= 3);
+        smooth_ms = sm_atof (args[1].c_str());
+        assert (smooth_ms > 10 && smooth_ms < 5000);
+        smooth_percent = sm_atof (args[2].c_str());
+        assert (smooth_percent > -1 && smooth_percent < 101);
+        return true;
+      }
+    return false;
+  }
+  void
+  usage (bool one_line)
+  {
+    printf ("<n_partials> <smooth_ms> <smooth_percent>\n");
+  }
+  bool
+  exec (Audio& audio)
+  {
+    AudioTool::auto_tune_smooth (audio, fundamental_est_n, smooth_ms, smooth_percent);
+    set_need_save (true);
 
-  for (size_t f = 0; f < audio.contents.size(); f++)
-    {
-      vector<uint16_t>& mags = audio.contents[f].mags;
-      for (size_t i = 0; i < mags.size(); i++)
-        mags[i] = sm_bound<int> (0, mags[i] + norm_delta_idb, 65535);
-
-      vector<uint16_t>& noise = audio.contents[f].noise;
-      for (size_t i = 0; i < noise.size(); i++)
-        noise[i] = sm_bound<int> (0, noise[i] + norm_delta_idb, 65535);
-    }
-
-  // store normalization in order to replay original samples normalized
-  const double samples_factor = db_to_factor (audio.original_samples_norm_db);
-  audio.original_samples_norm_db = db_from_factor (samples_factor * norm, -200);
-}
+    return true;
+  }
+} smooth_tune_command;
 
 static void
 normalize_energy (double energy, Audio& audio)
@@ -1049,7 +1025,7 @@ normalize_energy (double energy, Audio& audio)
   sm_printf ("avg_energy: %.17g\n", energy);
   sm_printf ("norm:       %.17g\n", norm);
 
-  normalize_factor (norm, audio);
+  AudioTool::normalize_factor (norm, audio);
 }
 
 class AutoVolumeCommand : public Command
@@ -1065,7 +1041,7 @@ public:
   {
     if (args.size() == 1)
       {
-        percent = atof (args[0].c_str());
+        percent = sm_atof (args[0].c_str());
         if (percent < 0 || percent > 100)
           {
             fprintf (stderr, "bad volume percentage: %f\n", percent);
@@ -1118,7 +1094,7 @@ public:
   {
     if (args.size() == 1)
       {
-        norm_db = atof (args[0].c_str());
+        norm_db = sm_atof (args[0].c_str());
         return true;
       }
     return false;
@@ -1131,7 +1107,7 @@ public:
   bool
   exec (Audio& audio)
   {
-    normalize_factor (db_to_factor (norm_db), audio);
+    AudioTool::normalize_factor (db_to_factor (norm_db), audio);
     return true;
   }
 } global_volume_command;
@@ -1147,7 +1123,7 @@ public:
   bool
   exec (Audio& audio)
   {
-    /* it would be nice if we could have options like --keep-samples || --strip-lpc
+    /* it would be nice if we could have options like --keep-samples
      * but for now we just do the default thing
      */
     for (size_t i = 0; i < audio.contents.size(); i++)
@@ -1160,24 +1136,64 @@ public:
   }
 } strip_command;
 
-class LPCCommand : public Command
+class StripAllCommand : public Command
 {
-  int lpc_order;
 public:
-  LPCCommand() : Command ("lpc")
+  StripAllCommand() : Command ("strip-all")
   {
     set_need_save (true);
   }
   bool
+  exec (Audio& audio)
+  {
+    for (size_t i = 0; i < audio.contents.size(); i++)
+      {
+        audio.contents[i].phases.clear();
+        audio.contents[i].debug_samples.clear();
+        audio.contents[i].original_fft.clear();
+      }
+    int loop_end = -1;
+    switch (audio.loop_type)
+      {
+        case Audio::LOOP_FRAME_FORWARD:
+        case Audio::LOOP_FRAME_PING_PONG:
+          /* strip frames */
+          loop_end = audio.loop_end;
+          break;
+
+        case Audio::LOOP_NONE:
+        case Audio::LOOP_TIME_FORWARD:
+        case Audio::LOOP_TIME_PING_PONG:
+          /* don't strip frames */
+          break;
+      }
+    if (loop_end >= 0)
+      {
+        size_t new_size = loop_end + 1;
+
+        if (new_size < audio.contents.size())
+          audio.contents.resize (new_size);
+      }
+    audio.original_samples.clear();
+    return true;
+  }
+} strip_all_command;
+
+class ExtractSMCommand : public Command
+{
+  int note;
+  string filename;
+public:
+  ExtractSMCommand() : Command ("extract-sm")
+  {
+  }
+  bool
   parse_args (vector<string>& args)
   {
-    if (args.size() == 1)
+    if (args.size() == 2)
       {
-        lpc_order = atoi (args[0].c_str());
-
-        assert ((lpc_order & 1) == 0);  // lpc order must be even
-        assert (lpc_order >= 2);        // ... and at least 2
-
+        note = atoi (args[0].c_str());
+        filename = args[1];
         return true;
       }
     return false;
@@ -1185,54 +1201,25 @@ public:
   void
   usage (bool one_line)
   {
-    printf ("<lpc_order>\n");
+    printf ("<note> <filename>\n");
   }
   bool
   exec (Audio& audio)
   {
-    const size_t frame_size = audio.frame_size_ms * LPC::MIX_FREQ / 1000;
-
-    for (AudioBlock& audio_block : audio.contents)
+    const WavSetWave *w = wave();
+    if (w && w->midi_note == note)
       {
-        AlignedArray<float,16> signal (frame_size);
-        for (size_t i = 0; i < audio_block.freqs.size(); i++)
-          {
-            const double freq = audio_block.freqs_f (i) * audio.fundamental_freq;
-            const double mag = audio_block.mags_f (i);
-
-            if (freq < LPC::MIX_FREQ / 2) // ignore freqs > nyquist
-              {
-                VectorSinParams params;
-                params.mix_freq = LPC::MIX_FREQ;
-                params.freq = freq;
-                params.phase = 0;
-                params.mag = mag;
-                params.mode = VectorSinParams::ADD;
-
-                fast_vector_sinf (params, &signal[0], &signal[frame_size]);
-              }
-          }
-        vector<double> lpc (lpc_order);
-
-        LPC::compute_lpc (lpc, &signal[0], &signal[frame_size]);
-
-        // make LPC filter stable
-        vector< complex<double> > roots;
-        bool good_roots = LPC::find_roots (lpc, roots);
-        assert (good_roots);
-        LPC::make_stable_roots (roots);
-        LPC::roots2lpc (roots, lpc);
-
-        LPC::lpc2lsf (lpc, audio_block.lpc_lsf_p, audio_block.lpc_lsf_q);
+        printf ("saving note %d to %s\n", w->midi_note, filename.c_str());
+        audio.save (filename);
       }
     return true;
   }
-} lpc_command;
+} extract_sm_command;
 
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   if (argc < 3)
     {
@@ -1271,7 +1258,7 @@ main (int argc, char **argv)
     {
       wav_set = new WavSet;
       Error error = wav_set->load (argv[1]);
-      if (error != 0)
+      if (error)
         {
           fprintf (stderr, "smtool: can't load file: %s\n", argv[1]);
           return 1;
@@ -1337,15 +1324,23 @@ main (int argc, char **argv)
     }
   if (need_save)
     {
-      if (audio && audio->save (argv[1]) != Error::NONE)
+      if (audio)
         {
-          fprintf (stderr, "error saving audio file: %s\n", argv[1]);
-          return 1;
+          Error error = audio->save (argv[1]);
+          if (error)
+            {
+              fprintf (stderr, "error saving audio file: %s\n", argv[1]);
+              return 1;
+            }
         }
-      if (wav_set && wav_set->save (argv[1]) != Error::NONE)
+      if (wav_set)
         {
-          fprintf (stderr, "error saving wavset file: %s\n", argv[1]);
-          return 1;
+          Error error = wav_set->save (argv[1]);
+          if (error)
+            {
+              fprintf (stderr, "error saving wavset file: %s\n", argv[1]);
+              return 1;
+            }
         }
     }
   if (wav_set)
