@@ -5,7 +5,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <assert.h>
-#include <sys/time.h>
 
 #include "config.h"
 #include <smwavset.hh>
@@ -15,6 +14,8 @@
 #include "smjobqueue.hh"
 #include "smutils.hh"
 #include "smwavdata.hh"
+#include "sminstrument.hh"
+#include "smwavsetbuilder.hh"
 
 #include <string>
 #include <map>
@@ -58,8 +59,9 @@ struct Options
   int             max_velocity;
   vector<string>  format;
   int             max_jobs;
-  bool            loop_markers;
-  enum { NONE, INIT, ADD, LIST, ENCODE, DECODE, DELTA, LINK, EXTRACT, GET_MARKERS, SET_MARKERS, SET_NAMES, GET_NAMES } command;
+  bool            loop_markers = false;
+  bool            loop_markers_ms = false;
+  enum { NONE, INIT, ADD, LIST, ENCODE, DECODE, DELTA, LINK, EXTRACT, GET_MARKERS, SET_MARKERS, SET_NAMES, GET_NAMES, BUILD } command;
 
   Options ();
   void parse (int *argc_p, char **argv_p[]);
@@ -92,8 +94,6 @@ Options::parse (int   *argc_p,
   gchar **argv = *argv_p;
   unsigned int i, e;
 
-  g_return_if_fail (argc >= 0);
-
   /*  I am tired of seeing .libs/lt-gst123 all the time,
    *  but basically this should be done (to allow renaming the binary):
    *
@@ -153,6 +153,10 @@ Options::parse (int   *argc_p,
         {
           loop_markers = true;
         }
+      else if (check_arg (argc, argv, &i, "--loop-ms"))
+        {
+          loop_markers_ms = true;
+        }
     }
 
   bool resort_required = true;
@@ -222,6 +226,10 @@ Options::parse (int   *argc_p,
             {
               command = GET_NAMES;
             }
+          else if (strcmp (argv[1], "build") == 0)
+            {
+              command = BUILD;
+            }
 
           if (command != NONE)
             {
@@ -248,6 +256,7 @@ Options::print_usage ()
   sm_printf (" smwavset link [ <options> ] <wset_filename>\n");
   sm_printf (" smwavset get-markers [ <options> ] <wset_filename>\n");
   sm_printf (" smwavset set-markers [ <options> ] <wset_filename> <marker_filename>\n");
+  sm_printf (" smwavset build <inst_filename> <smset_filename>\n");
   sm_printf ("\n");
   sm_printf ("options:\n");
   sm_printf (" -h, --help                  help for %s\n", options.program_name.c_str());
@@ -285,15 +294,6 @@ find_wave (WavSet& wset, int midi_note)
   return wi;
 }
 
-double
-gettime ()
-{
-  timeval tv;
-  gettimeofday (&tv, 0);
-
-  return tv.tv_sec + tv.tv_usec / 1000000.0;
-}
-
 string
 time2str (double t)
 {
@@ -341,9 +341,9 @@ void
 load_or_die (WavSet& wset, string name)
 {
   Error error = wset.load (name);
-  if (error != 0)
+  if (error)
     {
-      fprintf (stderr, "%s: can't open input file: %s: %s\n", options.program_name.c_str(), name.c_str(), sm_error_blurb (error));
+      fprintf (stderr, "%s: can't open input file: %s: %s\n", options.program_name.c_str(), name.c_str(), error.message());
       exit (1);
     }
 }
@@ -351,9 +351,9 @@ load_or_die (WavSet& wset, string name)
 int
 main (int argc, char **argv)
 {
-  double start_time = gettime();
+  double start_time = get_time();
 
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   options.parse (&argc, &argv);
 
@@ -444,7 +444,7 @@ main (int argc, char **argv)
         {
           string smpath = options.data_dir + "/" + int2str (wi->midi_note) + ".sm";
           string cmd = options.smenc + " -m " + int2str (wi->midi_note) + " \"" + wi->path.c_str() + "\" " + smpath + " " + options.args;
-          sm_printf ("[%s] ## %s\n", time2str (gettime() - start_time).c_str(), cmd.c_str());
+          sm_printf ("[%s] ## %s\n", time2str (get_time() - start_time).c_str(), cmd.c_str());
           job_queue.run (cmd);
 
           WavSetWave new_wave = *wi;
@@ -468,7 +468,7 @@ main (int argc, char **argv)
       for (vector<WavSetWave>::const_iterator si = smset.waves.begin(); si != smset.waves.end(); si++)
         {
           Audio audio;
-          if (audio.load (si->path, AUDIO_SKIP_DEBUG) != Error::NONE)
+          if (!audio.load (si->path, AUDIO_SKIP_DEBUG))
             {
               sm_printf ("can't load %s\n", si->path.c_str());
               exit (1);
@@ -476,7 +476,7 @@ main (int argc, char **argv)
 
           string wavpath = options.data_dir + "/" + int2str (si->midi_note) + ".wav";
           string cmd = "smplay --rate=" + int2str (audio.mix_freq) + " " +si->path.c_str() + " --export " + wavpath + " " + options.args;
-          sm_printf ("[%s] ## %s\n", time2str (gettime() - start_time).c_str(), cmd.c_str());
+          sm_printf ("[%s] ## %s\n", time2str (get_time() - start_time).c_str(), cmd.c_str());
           int rc = system (cmd.c_str());
           if (rc != 0)
             {
@@ -587,6 +587,14 @@ main (int argc, char **argv)
               sm_printf ("set-marker loop-end %d %d %d %d %d\n", wi->midi_note, wi->channel,
                 wi->velocity_range_min, wi->velocity_range_max, wi->audio->loop_end);
             }
+          if (options.loop_markers_ms)
+            {
+              const double zero_values_ms = wi->audio->zero_values_at_start / wi->audio->mix_freq * 1000.0;
+              const double loop_start     = wi->audio->loop_start * wi->audio->frame_step_ms - zero_values_ms;
+              const double loop_end       = wi->audio->loop_end * wi->audio->frame_step_ms - zero_values_ms;
+
+              sm_printf ("set-loop-ms %d %s %f %f\n", wi->midi_note, loop_type.c_str(), loop_start, loop_end);
+            }
         }
     }
   else if (options.command == Options::SET_MARKERS)
@@ -659,9 +667,30 @@ main (int argc, char **argv)
         }
       wset.save (argv[1]);
     }
+  else if (options.command == Options::BUILD)
+    {
+      assert (argc == 3);
+
+      string inst_filename = argv[1];
+      string out_filename = argv[2];
+
+      Instrument inst;
+      Error error = inst.load (inst_filename);
+      if (error)
+        {
+          fprintf (stderr, "%s: can't open input file: %s: %s\n", options.program_name.c_str(), inst_filename.c_str(), error.message());
+          exit (1);
+        }
+
+      WavSetBuilder builder (&inst, /* keep_samples */ false);
+      std::unique_ptr<WavSet> smset (builder.run());
+      assert (smset);
+
+      smset->save (out_filename);
+    }
   else
     {
-      sm_printf ("You need to specify a command (init, add, list, encode, decode, delta).\n\n");
+      sm_printf ("You need to specify a command (init, add, list, encode, decode, delta, build).\n\n");
       Options::print_usage();
       exit (1);
     }
