@@ -23,13 +23,13 @@ run (ADSREnvelope& adsr_envelope, size_t samples)
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   float rate = 48000;
 
   ADSREnvelope adsr_envelope;
 
-  adsr_envelope.set_config (atof (argv[1]), atof (argv[2]), atof (argv[3]), atof (argv[4]), rate);
+  adsr_envelope.set_config (sm_atof (argv[1]), sm_atof (argv[2]), sm_atof (argv[3]), sm_atof (argv[4]), rate);
   adsr_envelope.retrigger();
   run (adsr_envelope, sm_round_positive (rate / 2));
   adsr_envelope.release();
