@@ -2,6 +2,8 @@
 
 #include "smmidisynth.hh"
 #include "smmain.hh"
+#include "smproject.hh"
+#include "smsynthinterface.hh"
 
 using namespace SpectMorph;
 
@@ -10,16 +12,17 @@ using std::vector;
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
   if (argc != 3)
     {
       fprintf (stderr, "usage: testmidisynth <plan> <note>\n");
       return 1;
     }
 
+  Project   project;
   MidiSynth midi_synth (48000, 16 /* voices */);
 
-  MorphPlanPtr plan (new MorphPlan());
+  MorphPlanPtr plan (new MorphPlan (project));
 
   GenericIn *in = StdioIn::open (argv[1]);
   if (!in)
