@@ -5,20 +5,11 @@
 #include "smmorphplansynth.hh"
 #include "smmorphoutputmodule.hh"
 #include "smmain.hh"
-
-#include <sys/time.h>
+#include "smproject.hh"
+#include "smsynthinterface.hh"
 
 using namespace SpectMorph;
 
-static double
-gettime()
-{
-  timeval tv;
-  gettimeofday (&tv, 0);
-
-  return tv.tv_sec + tv.tv_usec / 1000000.0;
-}
-
 static void
 preinit_plan (MorphPlanPtr plan)
 {
@@ -36,10 +27,10 @@ measure_update (MorphPlanPtr plan, size_t n_voices)
 
   size_t runs = 1000000 / n_voices;
 
-  double start = gettime();
+  double start = get_time();
   for (size_t j = 0; j < runs; j++)
     synth.update_plan (plan);
-  double end = gettime();
+  double end = get_time();
 
   printf ("update (%zd voices): %f updates per ms\n", n_voices, 1 / ((end - start) * 1000 / runs));
 }
@@ -47,14 +38,15 @@ measure_update (MorphPlanPtr plan, size_t n_voices)
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
   if (argc != 2)
     {
       printf ("usage: %s <plan>\n", argv[0]);
       exit (1);
     }
 
-  MorphPlanPtr plan = new MorphPlan();
+  Project      project;
+  MorphPlanPtr plan = new MorphPlan (project);
   GenericIn *in = StdioIn::open (argv[1]);
   if (!in)
     {
