@@ -3,34 +3,25 @@
 #include "smrandom.hh"
 #include "smfft.hh"
 #include "smmain.hh"
+#include "smutils.hh"
 
 #include <glib.h>
 
-#include <sys/time.h>
 #include <stdio.h>
 #include <string>
 
 using namespace SpectMorph;
 using std::string;
 
-static double
-gettime()
-{
-  timeval tv;
-  gettimeofday (&tv, 0);
-
-  return tv.tv_sec + tv.tv_usec / 1000000.0;
-}
-
 int
 main (int argc, char **argv)
 {
   SpectMorph::Random random;
 
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   double clocks_per_sec = 2500.0 * 1000 * 1000;
-  double start = gettime();
+  double start = get_time();
 
   const int runs = 500000;
   const int bs = 1024;
@@ -42,10 +33,10 @@ main (int argc, char **argv)
       for (int b = 0; b < bs; b++)
         block_b[b] = random.random_uint32();
     }
-  double end = gettime();
+  double end = get_time();
   printf ("%f clocks/value\n", clocks_per_sec * (end - start) / runs / bs);
 
-  start = gettime();
+  start = get_time();
   for (int i = 0; i < runs; i++)
     {
       guint32 block[(bs + 3) / 4];
@@ -58,7 +49,7 @@ main (int argc, char **argv)
         }
 #endif
     }
-  end = gettime();
+  end = get_time();
 
   printf ("%f clocks/value\n", clocks_per_sec * (end - start) / runs / bs);
 }
