@@ -18,6 +18,7 @@
 #include "smwavset.hh"
 #include "smutils.hh"
 #include "smwavdata.hh"
+#include "sminstrument.hh"
 #include <stdlib.h>
 
 #if 1
@@ -29,7 +30,15 @@ debug (const char *fmt, ...)
 #define debug printf
 #endif
 
-using namespace SpectMorph;
+/* avoid problems with SpectMorph::Instrument and SpectMorph::Sample */
+using SpectMorph::GenericIn;
+using SpectMorph::JobQueue;
+using SpectMorph::WavSet;
+using SpectMorph::WavSetWave;
+using SpectMorph::WavData;
+using SpectMorph::Main;
+
+using SpectMorph::string_printf;
 
 using std::string;
 using std::vector;
@@ -37,6 +46,16 @@ using std::map;
 using std::sort;
 using std::set;
 
+static string
+get_current_dir()
+{
+  char *dir = g_get_current_dir();
+  string dir_str = dir;
+  g_free (dir);
+
+  return dir_str;
+}
+
 enum {
   GEN_PAN            = 17,
   GEN_INSTRUMENT     = 41,
@@ -504,6 +523,7 @@ struct Options
   bool                fast_import;
   bool                debug;
   bool                mono_flat;
+  bool                sminst = false;
   int                 max_jobs;
   string              config_filename;
   string              smenc;
@@ -536,8 +556,6 @@ Options::parse (int   *argc_p,
   gchar **argv = *argv_p;
   unsigned int i, e;
 
-  g_return_if_fail (argc >= 0);
-
   for (i = 1; i < argc; i++)
     {
       const char *opt_arg;
@@ -584,6 +602,10 @@ Options::parse (int   *argc_p,
         {
           mono_flat = true;
         }
+      else if (check_arg (argc, argv, &i, "--sminst"))
+        {
+          sminst = true;
+        }
     }
 
   bool resort_required = true;
@@ -977,6 +999,13 @@ import_preset (const string& import_name)
 {
   map<string,bool> is_encoded;
 
+  struct LoopRange
+  {
+    double  start  = -1;
+    double  end    = -1;
+  };
+  map<string,LoopRange> loop_range;
+
   for (vector<Preset>::iterator pi = presets.begin(); pi != presets.end(); pi++)
     {
       if (pi->name == import_name)
@@ -1093,6 +1122,10 @@ import_preset (const string& import_name)
                                                                   samples[id].endloop - samples[id].start +
                                                                   loop_shift);
 
+                                      // store loop range for SpectMorph::Instrument
+                                      loop_range[filename].start = (samples[id].startloop - samples[id].start + loop_shift) * 1000.0 / samples[id].srate;
+                                      loop_range[filename].end   = (samples[id].endloop - samples[id].start + loop_shift) * 1000.0 / samples[id].srate;
+
                                       // 200 ms padding at the end of the loop, to ensure that silence after sample
                                       // is not encoded by encoder
                                       padded_len = samples[id].end - samples[id].start + 0.2 * samples[id].srate;
@@ -1111,7 +1144,7 @@ import_preset (const string& import_name)
                                       padded_sample.assign (&sample_data[samples[id].start], &sample_data[samples[id].end]);
                                     }
                                   assert (padded_len == padded_sample.size());
-                                  WavData wav_data (padded_sample, 1, samples[id].srate);
+                                  WavData wav_data (padded_sample, 1, samples[id].srate, 16);
                                   if (!wav_data.save (filename))
                                     {
                                       fprintf (stderr, "%s: export to file %s failed: %s\n", options.program_name.c_str(), filename.c_str(), wav_data.error_blurb());
@@ -1134,25 +1167,64 @@ import_preset (const string& import_name)
                                 }
                               WavSetWave new_wave;
                               new_wave.midi_note = midi_note;
-                              new_wave.path = smname;
                               new_wave.channel = channel;
                               new_wave.velocity_range_min = vr_min;
                               new_wave.velocity_range_max = vr_max;
 
+                              if (options.sminst)
+                                new_wave.path = filename;
+                              else
+                                new_wave.path = smname;
+
                               wav_set.waves.push_back (new_wave);
                             }
                         }
                     }
                 }
             }
-          run_all (enc_commands, "Encoder", options.max_jobs);
-          run_all (strip_commands, "Strip", options.max_jobs);
+          if (options.sminst)
+            {
+              make_mono_flat (wav_set);
 
-          if (options.mono_flat)
-            make_mono_flat (wav_set);
+              SpectMorph::Instrument sminst;
 
-          wav_set.save (output_filename);
-          xsystem (string_printf ("smwavset link %s", output_filename.c_str()));
+              for (auto w : wav_set.waves)
+                {
+                  string path = get_current_dir() + "/" + w.path;
+
+                  SpectMorph::Sample *sample;
+                  SpectMorph::Error error = sminst.add_sample (path, &sample);
+                  if (sample)
+                    {
+                      sample->set_midi_note (w.midi_note);
+
+                      auto r = loop_range[w.path];
+                      if (r.start >= 0 && r.end >= 0)
+                        {
+                          sample->set_loop (SpectMorph::Sample::Loop::FORWARD);
+                          sample->set_marker (SpectMorph::MARKER_LOOP_START, r.start);
+                          sample->set_marker (SpectMorph::MARKER_LOOP_END,   r.end);
+                        }
+                    }
+                  else
+                    {
+                      fprintf (stderr, "%s: loading file '%s' failed: %s\n", options.program_name.c_str(), w.path.c_str(), error.message());
+                      exit (1);
+                    }
+                }
+              sminst.save (output_filename);
+            }
+          else
+            {
+              run_all (enc_commands, "Encoder", options.max_jobs);
+              run_all (strip_commands, "Strip", options.max_jobs);
+
+              if (options.mono_flat)
+                make_mono_flat (wav_set);
+
+              wav_set.save (output_filename);
+              xsystem (string_printf ("smwavset link %s", output_filename.c_str()));
+            }
         }
     }
   return 0;
@@ -1223,7 +1295,7 @@ dump (const string& preset_name = "")
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
   options.parse (&argc, &argv);
 
   if (options.command == Options::LIST)
