@@ -12,10 +12,7 @@
 using std::string;
 using std::vector;
 
-using SpectMorph::sm_init;
-using SpectMorph::sm_printf;
-using SpectMorph::WavData;
-using SpectMorph::Error;
+using namespace SpectMorph;
 
 /// @cond
 struct Options
@@ -27,7 +24,7 @@ struct Options
 int
 main (int argc, char **argv)
 {
-  sm_init (&argc, &argv);
+  Main main (&argc, &argv);
 
   options.program_name = "wav2ascii";
 
@@ -39,14 +36,23 @@ main (int argc, char **argv)
 
   /* open input */
   WavData wav_data;
-  if (!wav_data.load_mono (argv[1]))
+  if (!wav_data.load (argv[1]))
     {
       fprintf (stderr, "%s: can't open the input file %s: %s\n", options.program_name.c_str(), argv[1], wav_data.error_blurb());
       exit (1);
     }
 
-  for (size_t pos = 0; pos < wav_data.n_values(); pos++)
+  uint n_channels = wav_data.n_channels();
+  if (n_channels != 1 && n_channels != 2)
     {
-      sm_printf ("%.17g\n", wav_data[pos]);
+      fprintf (stderr, "%s: input file %s: only mono or stereo files are supported\n", options.program_name.c_str(), argv[1]);
+      exit (1);
+    }
+  for (size_t pos = 0; pos < wav_data.n_values(); pos += n_channels)
+    {
+      if (n_channels == 1)
+        sm_printf ("%.17g\n", wav_data[pos]);
+      else
+        sm_printf ("%.17g %.17g\n", wav_data[pos], wav_data[pos + 1]);
     }
 }
